///////////////////////////////////////////////////////////////////////////////
//                                                                           //
//                         Zonerings About box - Cool !                      //
//                             John Biddiscombe                             //
//                           J.Biddiscombe@rl.ac.uk                          //
//                                  Freeware                                 //
//                                                                           //
///////////////////////////////////////////////////////////////////////////////
unit AboutForm;

interface

uses
  { Borland }
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  Buttons, StdCtrls, ExtCtrls,
  { Mine }
  Math_utils, thinkbox, DIB_surf, Palunit, D_timer;

type
  TAbout_Form = class(TForm)
    Panel1: TPanel;
    Panel3: TPanel;
    DIBpanel: TPanel;
    persp: TScrollBar;
    y_rot: TScrollBar;
    x_rot: TScrollBar;
    z_rot: TScrollBar;
    OK_btn: TSpeedButton;
    Panel5: TPanel;
    Label1: TLabel;
    Label2: TLabel;
    Label3: TLabel;
    Label4: TLabel;
    Panel2: TPanel;
    about_speed: TPanel;
    cycle_speed: TScrollBar;
    morph_sc: TScrollBar;
    pause_sc: TScrollBar;
    Button1: TButton;
    Panel4: TPanel;
    procedure OK_btnClick(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure FormHide(Sender: TObject);
    procedure x_rotChange(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure morph_scChange(Sender: TObject);
    procedure pause_scChange(Sender: TObject);
    procedure Button1Click(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
    DIB_surface : DIBSurfaceObject;
    dest_DC     : hDC;
    frame_sp    : frame_speed_obj;
    function  GetPalette : hPalette; override;
    procedure setup_morphing_constants(steps:integer);
    procedure Setup_graphics;
    procedure initialize;
    procedure compute_and_draw_DIB(frame,stepnum:integer; framechange,skip:boolean);
    procedure animateflakes;
    procedure IdleAction(Sender:TObject; var Done:Boolean);
  end;

var
  About_Form: TAbout_Form;

var
  { screen variables }
  zone_DC          : HDC;
  xmid,ymid        : integer;
  { animation control }
  morph_sp         : integer;
  inter_frame      : integer;
  bigloop          : integer;
  framepause       : integer;
  frameloop        : integer;
  framenum         : integer;

var
  viewdist                   : double;
  xrotspeed,yrotspeed,zrotspeed : integer;
  xrotoff,yrotoff,zrotoff       : integer;
  xr,yr,zr                   : integer;
  xo,yo,zo                   : double;
  xs,ys,zs                   : double;
  fr_last                    : double;
  fr_temp                    : double;

const
  sw        = 383;
  sh        = 23;
  maxframes = 5;  // 6 total
type
  pixelstring = array[0..maxframes,0..sw,0..sh] of byte;

var
  string3D_1 : pixelstring;

type
  pflakestruc = ^flakestruc;
  flakestruc  = record
    x,y,z    : double;
    c        : double;
    dx,dy,dz : double;
    dc       : double;
  end;

const
  maxpoints = 2047;

var
  flakearray : array[0..maxframes,0..maxpoints] of flakestruc;

implementation

uses Main_window;

{$R *.DFM}

function TAbout_Form.GetPalette : hPalette;
begin
  if (DIB_surface<>nil) and (DIB_surface.DIBhpalette<>0) then result := DIB_surface.DIBhpalette
  else if hPalApp<>0 then result := hPalApp
  else result := inherited GetPalette;
end;

procedure TAbout_Form.compute_and_draw_DIB(frame,stepnum:integer; framechange,skip:boolean);
var col           : integer;
    x1,y1         : word;
    xt,yt,zt      : double;
    tx,ty,tz,tc   : double;
    lp1           : integer;
begin
  for lp1:=0 to maxpoints do begin
    with flakearray[frame,lp1] do begin
      tx:=x+(dx*stepnum); ty:=y+(dy*stepnum); tz:=z+(dz*stepnum);
      xt := ((rotyxz0_0*tx)) +
            ((rotyxz0_1*ty)) +
            ((rotyxz0_2*tz)) + xo;
      yt := ((rotyxz1_0*tx)) +
            ((rotyxz1_1*ty)) +
            ((rotyxz1_2*tz)) + yo;
      zt := ((rotyxz2_1*tx)) +
            ((rotyxz2_2*ty)) +
            ((rotyxz2_3*tz)) + zo;
      x1 := Round((xt*viewdist)/ zt)+xmid;
      y1 := Round((yt*viewdist)/ zt)+ymid;
      tc := c+(dc*stepnum);
      if tc>9 then DIB_surface.Set_pixel(x1,y1,trunc(tc));  // col 10-246
    end;
  end;
  DIB_surface.copy_surface_to_screen(dest_DC);
end;

procedure TAbout_Form.animateflakes;
var framechange : boolean;
begin
  inc(bigloop,1); framechange:=false;

  if framepause>=0 then inc(framepause)
  else inc(frameloop);
  if framepause>=inter_frame then begin
    framepause :=-1;
    frameloop  := 1;
  end;
  if frameloop>=morph_sp then begin
    frameloop  := 0;
    framepause := 0;
    inc(framenum); framechange:=true;
    if framenum>maxframes then framenum:=0;
  end;
  xo       := 0;
  yo       := 0;
  xr       := (512 + 64*xrotoff + 4*xrotspeed*bigloop) and tablesize;
  yr       := (512 + 64*yrotoff + 4*yrotspeed*bigloop) and tablesize;
  zr       := (64*zrotoff + 4*zrotspeed*bigloop) and tablesize;
  yxz_rotation(xr,yr,zr);
  DIB_surface.clear_surface;
  compute_and_draw_DIB(framenum,frameloop,framechange,false);
end;

procedure TAbout_Form.initialize;
begin
  inter_frame   := 30; // time paused before morphing again
  morph_sp      := 21; // time spent morphing
  bigloop       := 0;   // main loop
  framepause    := 0;   // counter
  frameloop     := 0;   // counter
  framenum      := 0;   // counter
  xrotspeed     := 3;
  yrotspeed     := 4;
  zrotspeed     := 7;
  // these all get reset by scrollbars so value may be wrong
end;

procedure TAbout_Form.OK_btnClick(Sender: TObject);
begin
  ModalResult := mrOK;
end;

procedure TAbout_Form.FormCreate(Sender: TObject);
var charfile : file;
begin
  initialize;
  if fileexists(ExtractFilePath(Application.Exename)+'Zone_Rings_32_Binary.bin') then begin
    assignfile(charfile,ExtractFilePath(Application.Exename)+'Zone_Rings_32_Binary.bin');
    reset(charfile,1);
    blockread(charfile,string3D_1[0],sizeof(string3D_1[0]));
    blockread(charfile,string3D_1[1],sizeof(string3D_1[0]));
    blockread(charfile,string3D_1[2],sizeof(string3D_1[0]));
    closefile(charfile);
  end
  else showmessage('Can''t find binary data file'); 
  frame_sp := frame_speed_obj.Create;
end;

procedure TAbout_Form.FormDestroy(Sender: TObject);
begin
  frame_sp.Free;
end;

procedure TAbout_Form.FormShow(Sender: TObject);
begin
  CenterWindow(Handle);
  PatBlt(ZoneWindow.canvas.handle,0,0,ZoneWindow.width,ZoneWindow.height,BLACKNESS);
  DIB_surface := DIBSurfaceobject.Create(App_Palette,Point(DIBpanel.Width,DIBpanel.Height));
  dest_DC     := GetDC(DIBpanel.handle);

  SelectPalette(dest_DC,DIB_surface.DIBhpalette,false);
  RealizePalette(dest_DC);

  xmid := DIBpanel.Width div 2;
  ymid := DIBpanel.Height div 2;
  frame_sp.reset(10);
  // force settings to agree with scrollbars
  setup_graphics;
  morph_scChange(nil);
  x_rotChange(nil);
  pause_scChange(nil);
  ActiveControl := x_rot;
  Application.OnIdle := IdleAction;
end;

procedure TAbout_Form.FormHide(Sender: TObject);
begin
  DIB_surface.Free;
  DIB_surface:=nil;
  ReleaseDC(DIBpanel.handle,dest_DC);
  Application.OnIdle := Zonewindow.IdleAction;
end;

procedure TAbout_Form.setup_morphing_constants(steps:integer);
var lp1,lp2 : integer;
    f1,f2   : integer;
begin
  for lp1:= 0 to maxframes do begin
    f1 := lp1; f2 := (lp1+1) mod (maxframes+1);
    for lp2:=0 to maxpoints do begin
      with flakearray[f1,lp2] do begin
        dx := (flakearray[f2,lp2].x - x)/ steps;
        dy := (flakearray[f2,lp2].y - y)/ steps;
        dz := (flakearray[f2,lp2].z - z)/ steps;
        dc := (flakearray[f2,lp2].c - c)/ steps;
      end;
    end;
  end;
end;

procedure TAbout_Form.Setup_graphics;
var lp0,lp1,lp2   : integer;
    a,r,cc        : double;
    r1,r2,cp,sp   : double;
    i,j,n,m       : integer;
    points_here   : integer;
    pp_seg,seg    : integer;
    phi,theta     : double;
begin
  points_here := 0;
  a := 200;
  r := sw+sw/12.5;  // frames 0 2 4 on disk, 1 3 5 generated in a minute
  for lp0:=0 to 2 do begin
    points_here := 0;
    for lp1:=0 to sw do begin
      for lp2:=0 to sh do begin
        if string3D_1[lp0,(sw-lp1),lp2]<>0 then begin
          with flakearray[lp0*2,points_here] do begin
            x := cos(2*pi*(sw-lp1)/r)*cos(pi*((lp2-sh/2)/a));
            z := sin(2*pi*(sw-lp1)/r)*cos(pi*((lp2-sh/2)/a));
            y := sin(pi*((lp2-sh/2)/a));
            c := 10+(lp1 mod pal_colours);
          end;
          inc(points_here);
        end;
      end;                                                    
    end; // put the spare points on a tiny sphere
    if points_here<maxpoints then for lp1:=points_here to maxpoints do begin
      with flakearray[lp0*2,lp1] do begin
        theta := 2*pi*random(256)/255; phi := 2*pi*random(256)/255;
        x := 0.02*Cos(theta)*Cos(phi);
        z := 0.02*Sin(theta)*Cos(phi);
        y := 0.02*sin(phi);
        c := 9;
      end;
    end;
  end;
  // frame 1 - random cube
  for lp1:=0 to maxpoints do begin
    with flakearray[1,lp1] do begin
      x := (0.57*2)*random(256)/255 - 0.57;
      z := (0.57*2)*random(256)/255 - 0.57;
      y := (0.57*2)*random(256)/255 - 0.57;
      c := 10+pal_colours*(sqrt(x*x + y*y + z*z)/sqrt(sqr(0.57)*3));
    end;
  end;
  // frame 3 - two spheres
  points_here := 0;
  seg         := 32;
  pp_seg      := (maxpoints+1) div (seg*2);
  m           := 0;
  for lp1:=0 to seg-1 do begin
    phi := pi*(m+0.5)/seg;
    n   := 0;
    for lp2:=0 to pp_seg-1 do begin
      theta := 2*pi*(n+0.5)/pp_seg;
      with flakearray[3,points_here] do begin
        x     := cos(theta)*sin(phi);
        y     := sin(theta)*sin(phi);
        z     := cos(phi);
        c     := 10+pal_coloursm1*(theta/(2*pi));
        inc(points_here);
      end;
      with flakearray[3,points_here] do begin
        x     := 0.5*cos(theta)*sin(phi);
        z     := 0.5*sin(theta)*sin(phi);
        y     := 0.5*cos(phi);
        c     := 10+pal_coloursm1*(theta/(2*pi));
        inc(points_here);
      end;
      n := n + 1;
    end;
    m := m + 1;
  end;
  // frame 5 - torus * 2
  r1 := 0.75;
  r2 := 0.25;
  n := 64;
  m := (maxpoints+1) div (n*2);
  points_here := 0;
  for i:=0 to n-1 do begin
    cp := cos(2*pi*i/n);
    sp := sin(2*pi*i/n);
//    r2 := 0.2 + 0.05*cos(8*pi*i/n);  // warp
    for j:=0 to m-1 do begin
      with flakearray[5,points_here] do begin
        x := cp*(r1+r2*cos(2*pi*j/m));
        y := sp*(r1+r2*cos(2*pi*j/m));
        z := r2*Sin(2*Pi*j/m);
        c := 10+pal_coloursm1*((1+cos(2*pi*i/n))/2);
        inc(points_here);
      end;
      with flakearray[5,points_here] do begin
        x := cp*(r1+r2*cos(2*pi*j/m));
        z := sp*(r1+r2*cos(2*pi*j/m));
        y := r2*Sin(2*Pi*j/m);
        c := 10+pal_coloursm1*((1+cos(2*pi*i/n))/2);
        inc(points_here);
      end;
    end;
  end;
end;

procedure TAbout_Form.IdleAction(Sender:TObject; var Done:Boolean);
var anim_pal : palentries;
    temp     : integer;
begin
  while tick_seconds2<(1/30.5) do begin end; 
  stop_ticker;                                
  about_speed.Caption := FloatToStrF(frame_sp.update(1/tick_seconds),ffFixed,5,0);
  start_ticker;
  temp := cycle_speed.position;

  GetPaletteEntries(DIB_surface.DIBhpalette,10,pal_colours,anim_pal[10]);
  if temp>0 then begin
    animatepalette(DIB_surface.DIBhpalette,10,pal_colours-temp,@anim_pal[10+temp]);
    animatepalette(DIB_surface.DIBhpalette,10+pal_colours-temp,temp,@anim_pal[10]);
  end
  else if temp<0 then begin temp:=-temp;
    animatepalette(DIB_surface.DIBhpalette,10+temp,pal_colours-temp,@anim_pal[10]);
    animatepalette(DIB_surface.DIBhpalette,10,temp,@anim_pal[10+pal_colours-temp]);
  end;
  SelectPalette(dest_DC,DIB_surface.DIBhpalette,false);
  RealizePalette(dest_DC);

  animateflakes;
  Done := false;
end;

procedure TAbout_Form.x_rotChange(Sender: TObject);
begin
  xrotspeed     := (x_rot.position);
  yrotspeed     := (y_rot.position);
  zrotspeed     := (z_rot.position);
  zo            := 1.3 + persp.position/10;
  viewdist      := zo*100;
end;

procedure TAbout_Form.morph_scChange(Sender: TObject);
begin
  morph_sp      := 1+(morph_sc.position*10); // time spent morphing
  frameloop     := 0;
  setup_morphing_constants(morph_sp);
end;

procedure TAbout_Form.pause_scChange(Sender: TObject);
begin
  inter_frame   := pause_sc.position;
end;

procedure TAbout_Form.Button1Click(Sender: TObject);
begin
  x_rot.position       := 0;
  y_rot.position       := 20;
  z_rot.position       := 0;
  persp.position       := 5;
  cycle_speed.position := 5;
  pause_sc.position    := 250;
  morph_sc.position    := 10;
  bigloop              := 32;
  framenum             := 0;
  frameloop            := 0;
  framepause           := 0;
  x_rotChange(nil);
  pause_scChange(nil);
  morph_scChange(nil);
end;

end.
