///////////////////////////////////////////////////////////////////////////////
//                                                                           //
//                          Zonerings Load/Save form                         //
//                             John Biddiscombe                             //
//                           J.Biddiscombe@rl.ac.uk                          //
//                                  Freeware                                 //
//                                                                           //
///////////////////////////////////////////////////////////////////////////////
unit LoadSaveForm;

interface

uses
  { Borland }
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, Buttons, ComCtrls, Menus, ExtCtrls, Grids,
  { Mine }
  thinkbox,PalUnit,D_timer,Str_Util, MyStrGrid, My_LZH;

var App_dir : String;

type
  Zone_setting = class(TObject)
    control_strings : TStringList;
    constructor Create;
    destructor  Destroy; override;
    function    ReadText(source:TMemoryStream) : boolean;
    procedure   WriteText(dest:TMemoryStream);
    function    name : string;
    function    screensave : boolean;
    procedure   setname(newname:string);
    procedure   setscreensave(newval:boolean);
  end;

type
  load_save_mode = (LoadMode,ViewMode,SaveMode,MustSaveMode);

type
  TLoadSave_Form = class(TForm)
    Container       : TPanel;
    Ok_btn          : TSpeedButton;
    Can_btn         : TSpeedButton;
    num_settings    : TLabel;
    Info_label      : TLabel;
    Screensaver_pic : TImage;
    ViewButton      : TButton;
    TextView        : TMemo;
    index_panel     : TPanel;
    Titles          : THeaderControl;
    settings_list: My_str_grid;

    procedure Ok_btnClick(Sender: TObject);
    procedure Can_btnClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure Settings_listMouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure ViewButtonClick(Sender: TObject);
    procedure Settings_listDrawCell(Sender: TObject; Col, Row: Longint;
      Rect: TRect; State: TGridDrawState);
    procedure Settings_listSelectCell(Sender: TObject; Col, Row: Longint;
      var CanSelect: Boolean);
    procedure Settings_listKeyPress(Sender: TObject; var Key: Char);
    procedure settings_listValidate(Sender: TObject; var Value: string;
      var Validate: Boolean; ACol, ARow: Longint);
  private
    { Private declarations }
  public
    { Public declarations }
    Lines_in_file    : integer;
    zone_index       : TList;
    memory           : TMemoryStream;
    compressed_data  : My_LZH_Compressor;
    Mode             : load_save_mode;
    enter_flag       : boolean;
    stored_index     : integer;
    modifications    : boolean;
    procedure Cleanup;
    procedure Dump_settings_file;
    procedure Scan_settings_file;
    function  GetPalette : hPalette; override;
    function  select_random_setting(just_tagged:boolean) : integer;
    function  Load_setting : integer;
    procedure Save_setting;
    procedure setup_load_mode;
    procedure setup_view_mode;
    procedure setup_save_mode;
    procedure setup_mustsave_mode;
    procedure exit_editor;
    function  check_save : boolean;
    procedure overwrite_setting(newname:Pchar);
  end;

var
  LoadSave_Form : TLoadSave_Form;

implementation

uses SettingsForm, OptionsForm;

{$R *.DFM}

///////////////////////////////////////////////////////////////////////////////
//  Zone_setting methods
///////////////////////////////////////////////////////////////////////////////
constructor Zone_setting.Create;
begin
  control_strings := TStringList.Create;
end;

destructor Zone_setting.Destroy;
begin
  control_strings.Free;
end;

function Zone_setting.ReadText(source:TMemoryStream) : boolean;
var line : string;
    done : boolean;
begin
  control_strings.Clear;
  result := true;
  done   := false;
  line   := read_line(source);
  if line[1] = '{' then begin
    line[1]:=' '; line:=Trim(line);
    if line<>'' then control_strings.Add(line);
  end else result := false;
  if result then repeat
    line := read_line(source);
    if line='' then result:=false
    else if line[1] = '}' then done := true
    else control_strings.Add(line);
  until done or (not result);
end;

procedure Zone_setting.WriteText(dest:TMemoryStream);
var lp1 : integer;
begin
  WriteString(dest,'{ ');
  for lp1 := 0 to control_strings.count-1 do WriteString(dest,control_strings[lp1]);
  WriteString(dest,'}');
end;

function Zone_setting.name : string;
begin
  result := control_strings[0];
end;

function Zone_setting.screensave : boolean;
var lp1 : integer;
begin
  result := false;
  for lp1:=0 to control_strings.count-1 do begin
    if pos('Screensaver',control_strings[lp1])<>0 then begin
      if pos('TRUE',control_strings[lp1])<>0 then begin result:=true; exit; end
      else result:=false;
    end;
  end;
end;

procedure Zone_setting.setname(newname:string);
begin
  control_strings[0] := newname;
end;

procedure Zone_setting.setscreensave(newval:boolean);
var lp1 : integer;
    s   : string;
begin
  for lp1:=0 to control_strings.count-1 do begin
    if pos('Screensaver',control_strings[lp1])<>0 then begin
      control_strings[lp1] := Format('%20s',['Screensaver'])+#9+BoolToStr(newval);
      exit;
    end;
  end;
  control_strings.Add(Format('%20s',['Screensaver'])+#9+BoolToStr(newval));
end;

///////////////////////////////////////////////////////////////////////////////
//  Load_Save form event handlers
///////////////////////////////////////////////////////////////////////////////
procedure TLoadSave_Form.FormCreate(Sender: TObject);
begin
  App_dir    := ExtractFilePath(Application.Exename);

  compressed_data  := My_LZH_Compressor.Create;
  if (Options_Form.UseCompression.Checked) and FileExists(App_dir+Options_Form.Settings_file_name.Text+'.cmp') then begin
    thinking(nil,'Decompressing Zonerings settings data');
    compressed_data.o_s.LoadFromFile(App_dir+Options_Form.Settings_file_name.Text+'.cmp');
    compressed_data.compression(false);
  end
  else if FileExists(App_dir+Options_Form.Settings_file_name.Text+'.txt') then
    compressed_data.i_s.LoadFromFile(App_dir+Options_Form.Settings_file_name.Text+'.txt')
  else begin
    showmessage('Settings file was not saved in format set by INI options'#13#10+
                '          Attempting to load alternative file          ');
    halt;
  end;
  compressed_data.progressbar     := Thinking_box.bar;
  compressed_data.progressbar.max := trunc(compressed_data.i_s.Size/1024);
  memory     := compressed_data.i_s;
  zone_index := Tlist.Create;
  Scan_settings_file;
  if Options_Form.Run_mode.checked then select_random_setting(Options_Form.cycle_tagged.Checked)
  else select_random_setting(True);
  modifications := false;
  stopped_thinking;
end;

procedure TLoadSave_Form.Cleanup;
begin
  if modifications then begin
    if Options_Form.UseCompression.Checked then begin
      thinking_bar(nil,'Compressing Zonerings settings data');
      Dump_settings_file;
      compressed_data.compression(true);
      stopped_thinking;
      compressed_data.o_s.SaveToFile(App_dir+Options_Form.Settings_file_name.Text+'.cmp');
    end
    else compressed_data.i_s.SaveToFile(App_dir+Options_Form.Settings_file_name.Text+'.txt');
  end;
  compressed_data.free;
end;

procedure TLoadSave_Form.Dump_settings_file;
var lp1,lp2 : integer;
begin
  memory.Clear;
  for lp1:=0 to zone_index.count-1 do with Zone_setting(zone_index[lp1]) do WriteText(memory);
end;

procedure TLoadSave_Form.Scan_settings_file;
var entry   : Zone_setting;
    entries : integer;
begin
  memory.position := 0;
  repeat
    entry := Zone_setting.Create;
    if not entry.ReadText(memory) then begin
      Showmessage('The Settings file may be corrupt'+#13#10+'You may experience problems');
      entry.Free;
      break;
    end;
    entries := zone_index.Add(entry);
    settings_list.RowCount := entries+1;
    settings_list.cells[0,entries] := entry.name;
    settings_list.cells[1,entries] := '';
  until memory.position = memory.size;
end;

procedure TLoadSave_Form.FormShow(Sender: TObject);
begin
  CenterWindow(Handle);
  num_settings.Caption := 'Number of settings in file : ' + IntToStr(zone_index.count);
  with settings_list do begin
    if (Row-8>=0) and (Row+8<=Rowcount) then settings_list.TopRow := settings_list.Row-8
    else if (Row-8<0) then settings_list.TopRow := 0
    else if (Row+8>RowCount) then settings_list.TopRow := RowCount-15;
  end;
  if Mode=LoadMode then setup_load_mode
  else if Mode=SaveMode then setup_save_mode;
  ViewButton.Enabled    := true;
  with settings_list do begin
    Titles.Sections.Items[0].width:=width-Titles.Sections.Items[1].width-GetSystemMetrics(SM_CXVSCROLL);
    ColWidths[0] := Titles.Sections.Items[0].width-2;
    ColWidths[1] := Titles.Sections.Items[1].width-1;
    stored_index := Row;
  end;
end;

function TLoadSave_Form.select_random_setting(just_tagged:boolean) : integer;
var lp1,choice : integer;
begin
  result := -1;
  if just_tagged then begin
    for lp1:=0 to 5000 do begin   // keep selecting random settings until one with flag set
      choice := random(zone_index.count);
      if Zone_setting(zone_index[choice]).Screensave then begin
        result := choice;
        break;
      end;
    end;
  end;
  if result=-1 then result := random(zone_index.count);
  settings_list.row := result;
  Settings_Form.ReadText(Zone_setting(zone_index[result]).control_strings);
end;

procedure TLoadSave_Form.exit_editor;
begin
  if settings_list.EditorMode then begin
    settings_list.EnterFlag := true;
    SendMessage(settings_list.Editor.handle,CM_EXIT,0,0);
    SendMessage(settings_list.handle,CM_EXIT,0,0);
    settings_list.EditorMode := false;
  end;
  if Info_label.Caption  = 'You must press <Enter> to commit changes' then
    Info_label.Caption  := 'Double click on an area to edit';
  if (mode=LoadMode) then settings_list.options := settings_list.options - [goAlwaysShowEditor] - [goEditing]  + [goThumbTracking];
end;

procedure TLoadSave_Form.setup_load_mode;
begin
  Mode                := LoadMode;
  index_panel.Visible := true;
  TextView.Visible    := false;
  Info_label.Caption  := 'Double click on an area to edit';
  ViewButton.Caption  := 'View setting as text';
  settings_list.options := settings_list.options - [goAlwaysShowEditor] - [goEditing]  + [goThumbTracking];
  exit_editor;
end;

procedure TLoadSave_Form.setup_view_mode;
const ts : array[0..2] of integer = (64,64+32,64+32+32);
begin
  exit_editor;
  Mode                  := ViewMode;
  index_panel.Visible   := false;
  TextView.Visible      := true;
  Info_label.Caption    := 'You cannot modify values in this mode';
  ViewButton.Caption    := 'Return to index view';
  SendMessage(TextView.handle,EM_SetTabStops,3,integer(@ts));
  settings_list.options := settings_list.options - [goAlwaysShowEditor] - [goEditing]  + [goThumbTracking];
end;

procedure TLoadSave_Form.setup_save_mode;
begin
  Mode                := SaveMode;
  Info_label.Caption  := 'You will overwrite selected setting';
  ViewButton.Caption  := 'Create new setting';
  index_panel.Visible := true;
  TextView.Visible    := false;
  settings_list.options := settings_list.options + [goAlwaysShowEditor] + [goEditing] + [goThumbTracking];
  settings_list.EditorMode := True;
  settings_list.Editor.SelectAll;
  ActiveControl := settings_list.Editor;
end;
                                  // andy 726920
procedure TLoadSave_Form.setup_mustsave_mode;
begin
  Mode                  := MustSaveMode;
  Info_label.Caption    := 'Please enter a suitable (preferably silly) name';
  ViewButton.Enabled    := false;
  settings_list.options := settings_list.options + [goAlwaysShowEditor] + [goEditing] - [goThumbTracking];
end;

procedure TLoadSave_Form.Ok_btnClick(Sender:TObject);
begin
  if ((mode=SaveMode) or (mode=MustSaveMode)) then begin
    if (mode=SaveMode) and (not check_save) then exit;
    enter_flag:=true;
  end;
  exit_editor;
  if (Mode=LoadMode) or (Mode=ViewMode) then begin
    LockWindowUpdate(Settings_Form.handle);
    Settings_Form.ReadText(Zone_setting(zone_index[settings_list.row]).control_strings);
    LockWindowUpdate(0);
    ModalResult := mrOK;
  end else begin
    if (mode=SaveMode) then begin
      overwrite_setting(Pchar(settings_list.Cells[0,settings_list.Row]));
      ModalResult := mrOK;
    end
    else if (mode=MustSaveMode) then begin
      with settings_list do Zone_setting(zone_index[Rowcount-1]).SetName(Cells[0,RowCount-1]);
      ModalResult := mrOK;
    end;
  end;
end;

function TLoadSave_Form.check_save : boolean;
begin
  if (Mode=LoadMode) or (Mode=ViewMode) then exit;
  case MessageDlg('Overwrite'#13#13+Zone_setting(zone_index[settings_list.row]).name,
    mtConfirmation, [mbYes,mbNo], 0) of
    mrYes    : result := true; 
    mrNo     : result := false;
  end
end;

procedure TLoadSave_Form.Can_btnClick(Sender: TObject);
begin
  exit_editor;
  if mode=MustSaveMode then begin
    settings_list.Hide_editor;
    Zone_setting(Zone_index[Zone_index.count-1]).Free;
    Zone_index.delete(Zone_index.count-1);
    settings_list.RowCount := settings_list.RowCount-1;
    mode := LoadMode; // so we can select new row...
  end;
  settings_list.Row := stored_index;
  ModalResult := mrCancel;
end;

function TLoadSave_Form.GetPalette : hPalette;
begin
  if hPalApp<>0 then result := hPalApp
  else result := inherited GetPalette;
end;

procedure TLoadSave_Form.Settings_listKeyPress(Sender: TObject;
  var Key: Char);
begin
  if settings_list.EditorMode and settings_list.Editor.Visible and (key=#13) then with settings_list do begin
    enter_flag := true;
    if mode=(LoadMode) then begin
      Zone_setting(zone_index[Row]).SetName(Editor.Text);
      exit_editor;
    end;
    if Info_label.Caption  = 'You must press <Enter> to commit changes' then
      Info_label.Caption  := 'Double click on an area to edit';
    if (mode=SaveMode) then begin
      if check_save then begin
        overwrite_setting(Pchar(Editor.Text));
        ModalResult := mrOK;
      end else EditorMode := true;
    end
    else if (mode=MustSaveMode) then begin
      with settings_list do Zone_setting(zone_index[Rowcount-1]).SetName(Cells[0,RowCount-1]);
      ModalResult := mrOK;
    end;
  end;
end;

procedure TLoadSave_Form.Settings_listMouseDown(Sender: TObject;
  Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
var z_s : Zone_setting;
    c,r : longint;
begin
  enter_flag  := false;
  if (mode=LoadMode) then exit_editor;
  if not (ssDouble in shift) then exit;
  settings_list.MouseToCell(x,y,c,r);
  if (c=0) and not ((mode=SaveMode) or (mode=MustSaveMode)) then begin
    settings_list.Options := settings_list.Options + [goEditing];
    settings_list.EditorMode := true;
    info_label.Caption := 'You must press <Enter> to commit changes';
  end else if (c=1) then begin
    z_s := Zone_setting(zone_index[r]);
    z_s.SetScreensave(not z_s.Screensave);
    settings_list.repaint;
    modifications := true;
  end;
end;

procedure TLoadSave_Form.ViewButtonClick(Sender: TObject);
var astring : string;
    lp1     : integer;
    z_s     : Zone_setting;
begin
  if Mode=SaveMode then begin
    Settings_list.RowCount := Settings_list.RowCount+1;
    Settings_list.Cells[0,Settings_list.RowCount-1] := '';
    Settings_list.Row := Settings_list.RowCount-1;
    z_s := Zone_setting.Create;
    Settings_Form.WriteText(z_s.Control_strings,'');
    zone_index.Add(z_s);
    num_settings.Caption := 'Number of settings in file : ' + IntToStr(zone_index.count);
    setup_mustsave_mode;
    settings_list.EditorMode := True;
    settings_list.SetFocus;
    exit;
  end else if Mode=LoadMode then begin
    exit_editor;
    TextView.Lines.BeginUpdate;
    TextView.Lines.Clear;
    for lp1:=0 to Zone_setting(zone_index[settings_list.row]).control_strings.Count-1 do
      TextView.Lines.Add(Trim(Zone_setting(zone_index[settings_list.row]).control_strings[lp1]));
    TextView.Lines.Insert(1,' ');
    TextView.Lines.EndUpdate;
    setup_view_mode;
  end else if Mode=ViewMode then begin
    setup_Load_mode;
  end;
end;

function TLoadSave_Form.Load_setting : integer;
begin
  Mode := LoadMode;
  result := showmodal;
end;

procedure TLoadSave_Form.Save_setting;
begin
  Mode := SaveMode;
  showmodal;
end;

procedure TLoadSave_Form.Settings_listDrawCell(Sender: TObject; Col,
  Row: Longint; Rect: TRect; State: TGridDrawState);
var temp,sw : integer;
    z_s     : Zone_setting;
begin
  with (Sender as TStringGrid).Canvas do begin
    if Col=0 then begin
      if gdSelected in State then begin
        brush.color := clHighlight;
        font.color  := clHighlightText;
        font.style  := [fsBold];
      end else begin
        brush.color := clBtnFace;
        font.color  := clWindowText;
        font.style  := [fsBold];
      end;
      FillRect(Rect);
      TextOut(Rect.Left + 2, Rect.Top + 2, Settings_list.Cells[0,row]);
    end else begin
      sw := Titles.Sections.Items[1].left;
      z_s := Zone_setting(zone_index[row]);
      if z_s.screensave then Draw(Rect.Left+12, Rect.Top+2, Screensaver_pic.picture.bitmap)
      else begin brush.color:=clBtnFace; FillRect(Rect); end;
    end;
  end;
end;

procedure TLoadSave_Form.Settings_listSelectCell(Sender: TObject; Col,
  Row: Longint; var CanSelect: Boolean);
begin
  if Col<>0 then CanSelect := false;
  if (mode=MustSaveMode) then if Row<>settings_list.Rowcount-1 then CanSelect:=false;
end;

procedure TLoadSave_Form.overwrite_setting(newname:Pchar);
var temp     : TMemoryStream;
    inx      : integer;
    p1,p2    : integer;
    z_s     : Zone_setting;
begin
  z_s := Zone_setting.Create;
  Settings_Form.WriteText(z_s.Control_strings,'');
  z_s.SetName(string(newname));
  settings_list.Cells[0,settings_list.Row] := string(newname);
  Zone_setting(zone_index[settings_list.Row]).Free;
  zone_index[settings_list.Row] := z_s;
  modifications := true;
end;

procedure TLoadSave_Form.settings_listValidate(Sender: TObject;
  var Value: string; var Validate: Boolean; ACol, ARow: Longint);
begin
  if enter_flag {and (mode=LoadMode) }then begin
    validate := true;
    modifications := true;
  end
  else validate := false;
  enter_flag := false;
end;

end.
