///////////////////////////////////////////////////////////////////////////////
//                                                                           //
//                           Main Zonerings window                           //
//                             John Biddiscombe                             //
//                           J.Biddiscombe@rl.ac.uk                          //
//                                  Freeware                                 //
//                                                                           //
///////////////////////////////////////////////////////////////////////////////
unit Main_window;

interface

uses
  { Borland }
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, Math,
  { mine }
  Math_utils,PalUnit,thinkbox,D_timer;

const
  maximum_polys = 200;

type
  polypoints  =  array[0..10] of Tpoint;

type
  poly  = class(Tobject)
    col       : integer;    { these 8 must be kept together }
    vel       : integer;    { because I've block copied     }
    xc        : integer;    { them in in one go...          }
    yc        : integer;    { See routine copy_from_below   }
    sid       : integer;    {                               }
    changed   : longbool;   {                               }
    xasp      : double;     {                               }
    yasp      : double;     {                               }
    rad       : double;
    rot       : integer;
    vertices  : polypoints;
    constructor Create(x,y,s,c,rt,ve:integer; rd,xa,ya:double);
    procedure   erasevert;
    procedure   drawvert;
    procedure   erasepoly;
    procedure   drawpoly;
    procedure   copy_from_below(source:poly);
    procedure   update_vertices;
  end;

  ppolylist = ^polylist;
  polylist  = array[0..maximum_polys] of poly;

  zonerings  = class(Tobject)
    central_poly   : poly;
    num_polys      : integer;

    aspect_flag    : boolean;
    aspect_countx  : integer;
    aspect_county  : integer;
    aspect_incx    : integer;
    aspect_incy    : integer;

    shape_counter  : integer;
    shape_timeout  : integer;

    rotat_counter  : integer;
    rotat_timeout  : integer;
    rot_inc        : integer;
    rot_inc2       : integer;
    rot_loop       : integer;
    rot_ampl       : double;
    tot_rot        : integer;

    col_inc        : integer;
    col_max        : integer;

    mot_damping    : double;
    movex_timeout  : integer;
    movex_counter  : integer;
    movey_timeout  : integer;
    movey_counter  : integer;
    move_tempx     : integer;
    move_tempy     : integer;
    lastx,lasty    : integer;
    diffx,diffy    : integer;
    x_count        : integer;
    x_inc          : integer;
    x_count2       : integer;
    x_inc2         : integer;
    x_rad          : double;
    x_div          : integer;
    y_count        : integer;
    y_inc          : integer;
    y_count2       : integer;
    y_inc2         : integer;
    y_rad          : double;
    y_div          : integer;

    listpoly       : polylist;

    constructor Create;
    destructor  destroy;    override;
    procedure   draw_all_rings(adc:hDC);
    procedure   cyclerings;
    function    sides : word;
    procedure   setup_densities;
  end;

  TZonewindow = class(TForm)
    procedure FormResize(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormClick(Sender: TObject);
    procedure FormMouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure FormPaint(Sender: TObject);
    procedure FormMouseMove(Sender: TObject; Shift: TShiftState; X,
      Y: Integer);
    procedure FormDestroy(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
    speed_ok      : boolean;
    zone_instance : zonerings;
    frame_speed   : frame_speed_obj;
    want_speed    : double;
    one_second    : TDateTime;
    time_now      : TDatetime;
    last_time     : TDatetime;
    Procedure NewSetting;
    function  are_settings_different(CheckStrings:TStringList) : boolean;
    procedure IdleAction(Sender:TObject; var Done:Boolean);
    function  GetPalette : hPalette; override;
    procedure WMSysCommand(var message:TWMSysCommand); message WM_SysCommand; 
  end;

var
  Zonewindow       : TZonewindow; // main black full-screen form

  zone_DC          : hDC;
  App_pens         : pen_array;
  cols             : array[0..255] of TColorRef;

  xmid,ymid        : integer;
  screen_          : integer;
  x_asp,y_asp      : double;
  mouseX,mouseY    : integer;
  sigmax,sigmay    : integer;
  total_prob       : integer;          { all the densities }
  densities        : array[0..9] of word;
  paused           : boolean;
  dialog_active    : boolean;

  pal_pos          : integer;
  pal_inc          : integer;

implementation

uses OptionsForm, SettingsForm, LoadSaveForm, AboutForm;

{$R *.DFM}

{
// Routines used in (integer) fixed point version - not implemented here.

function Fixed16_16ToFloat(x:integer) : extended;
begin
  Fixed16_16ToFloat:=x/65536;
end;

function FloatToFixed16_16(x:extended) : integer;
begin
  FloatToFixed16_16:=round(x*65536);
end;

function Fixed16_16Mul(A,B:integer) : integer; register; assembler;
asm // A = EAX : B = EDX : C = ECX
  IMUL EDX;
  SHRD EAX,EDX,16;
end;
{
function Fixed16_16Mul3(A,B,C:integer) : integer; register; assembler;
asm // A = EAX : B = EDX : C = ECX
  IMUL EDX;
  SHRD EAX,EDX,16;
  IMUL ECX;
  SHRD EAX,EDX,16;
end;
}
{
function Zone_DC : hDC;
begin
  result := zonewindow.canvas.handle;
end;
}

function log(x:double):double;
begin
  if x>0 then log:=system.ln(x)/system.ln(10)
  else log:=-150;
end;

{ ----------------------------------------------------------------------- }
{                          Polygon object structure                       }
{ ----------------------------------------------------------------------- }
constructor poly.Create(x,y,s,c,rt,ve:integer; rd,xa,ya:double);
begin
  inherited Create;
  xc      := x;
  yc      := y;
  sid     := s;
  col     := c;
  rad     := rd;
  rot     := rt;
  vel     := ve;
  xasp    := xa;
  yasp    := ya;
  changed := false;
  update_vertices;
end;
{ ----------------------------------------------------------------------- }
procedure poly.update_vertices;
var lp1,theta,thetad : integer;
begin
  if sid>1 then begin thetad:=theta_d[sid];
    theta:=(rot shr 16) and tablesize;
    for lp1:=0 to sid-1 do begin
      vertices[lp1].x := round(xc+(cosine[theta]*xasp*rad));
      vertices[lp1].y := round(yc+(sine  [theta]*yasp*rad));
      theta           := (theta+thetad) and tablesize;
    end;
    vertices[sid].x := vertices[0].x;
    vertices[sid].y := vertices[0].y;
  end
  else begin
    theta:=(rot shr 16) and tablesize;
    vertices[0].x := (xc+round(cosine[theta]*xasp*rad));
    vertices[0].y := (yc+round(sine  [theta]*yasp*rad));
  end;
end;

procedure poly.erasevert;
var lp1 : integer;
begin
  for lp1:=0 to sid-1 do setpixelV(zone_DC,vertices[lp1].x,vertices[lp1].y,0);
end;

procedure poly.drawvert;
var lp1 : integer;
begin
//  for lp1:=0 to sid-1 do setpixelV(zone_DC,vertices[lp1].x,vertices[lp1].y,PaletteIndex((col shr 16)+10));
  for lp1:=0 to sid-1 do setpixelV(zone_DC,vertices[lp1].x,vertices[lp1].y,cols[((col shr 16)+10)and 65535]);
//  for lp1:=0 to sid-1 do setpixel(zone_DC,vertices[lp1].x,vertices[lp1].y,RGB(255,255,255));
end;

procedure poly.erasepoly;
begin
  SelectObject(zone_DC,App_pens[0]);
  if sid>2 then polyline(zone_DC,vertices,sid+1)
  else if sid=2 then polyline(zone_DC,vertices,sid)
  else setpixelV(zone_DC,vertices[0].x,vertices[0].y,0);
end;

procedure poly.drawpoly;
begin
  SelectObject(zone_DC,App_pens[((col shr 16)+10)and 65535]);
  if sid>2 then polyline(zone_DC,vertices,sid+1)
  else if sid=2 then polyline(zone_DC,vertices,sid)
//  else setpixel(zone_DC,vertices[0].x,vertices[0].y,RGB(255,255,255)
  else setpixelV(zone_DC,vertices[0].x,vertices[0].y,cols[((col shr 16)+10)and 65535] );
end;

type
  intarray = array[0..7] of integer;
  pintarray = ^intarray;

procedure poly.copy_from_below(source:poly);
begin
  with source do begin
    self.col     := col;
    self.vel     := vel;
    self.xc      := xc;
    self.yc      := yc;
    self.sid     := sid;
    self.changed := changed;
    self.xasp    := xasp;
    self.yasp    := yasp;
    self.rot     := rot+vel;
    changed      := false;
  end;
  update_vertices;
end;
{ ----------------------------------------------------------------------- }
{           Multiple Polygons (Zonerings) object structure                }
{ ----------------------------------------------------------------------- }
constructor zonerings.Create;
var lp1               : integer;
    rad               : double;
    rot,rat           : integer;
    xa,ya             : double;
    scale,dvd,dvd2    : double;
    x,y,off           : double;
begin
  with Settings_Form.palette_list do begin
    if ItemIndex<>-1 then
      create_256_identity_palette_from_file(App_Palette,hPalApp,Items[ItemIndex])
    else showmessage('Palette files not found in '+App_dir+'Palette');
  end;
  create_256_pens_from_palette(hPalApp,App_Palette,App_pens, 0);

  setup_densities;
  aspect_countx  := 0;
  aspect_county  := 0;
  with Settings_Form do begin
    if max_polygons.position>maximum_polys then num_polys:=maximum_polys
    else                                        num_polys:=max_polygons.position;
    off   := (vanishing.position) / (vanishing.max+1);
    dvd   := 4/magnification.max;
    scale := screen_*dvd*magnification.position;
    dvd   := (1-off)/num_polys;
    dvd2  := 5 / perspective.max;
    x_asp := ((aspect_a_x.position) / (aspect_a_x.max));
    y_asp := ((aspect_a_y.position) / (aspect_a_y.max));
    aspect_incx    := aspect_f_x.position*8;
    aspect_incy    := aspect_f_y.position*8;
    if (aspect_f_x.position<>0) or (aspect_f_y.position<>0) then aspect_flag:=true
    else aspect_flag := false;
    xa := x_asp;
    ya := y_asp;
    for lp1:=0 to num_polys do begin
      rot:=0; rat:=0;
      if in_out.checked then x := off + lp1*dvd
      else                   x := off + (num_polys-lp1)*dvd;
      if aspect_flag then begin
        xa    := 1-(x_asp*(0.5+(sine[aspect_countx] / 2)));
        ya    := 1-(y_asp*(0.5+(sine[aspect_county] / 2)));
      end;
      if perspective.position>=0 then begin
        y   := 1 + (dvd2*perspective.position);
        rad := round(powerxy(x,y)*scale);
      end else begin
        y   :=-1 + dvd2*perspective.position;
        rad := round((1-powerxy(1-x,-y))*scale);
      end;
      listpoly[lp1]:=poly.Create(xmid,ymid,sides,-655360, rot, rat, rad, xa, ya);
    end;

    listpoly[0].col := 65536*random(pal_colours);
    central_poly    := listpoly[0];

    if shape_per.checked then shape_timeout := shape_freq.max-shape_freq.position
    else                      shape_timeout := round(50*(2-Log(shape_freq.max-shape_freq.position)));
    shape_counter  := 0;

    rotat_counter  := 0;
    rot_inc        := (8*precess.position) shl 16;
    rot_inc2       := (4*freq.position);
    rot_loop       := 0;
    rot_ampl       := (2*ampl.position);
    tot_rot        := max_rot_speed.position-min_rot_speed.position;
    if rot_ran_per.checked then rotat_timeout := rot_ran_freq.max-rot_ran_freq.position
    else                        rotat_timeout := round(50*(2-Log(rot_ran_freq.max-rot_ran_freq.position)));
    if ran_rotation.checked then with central_poly do begin
      rot := random(tablesizep1) shl 16;
      vel := (random(tot_rot)+min_rot_speed.position)*(8+random(8)) shl 16;
    end;

    col_inc        := 6000*integer(colour_step.position);
    col_max        := 65536*pal_colours;

    movex_counter  := 0;
    movey_counter  := 0;
    diffx          := 0;      diffy         := 0;
    mouseX         := xmid;   mouseY        := ymid;
    if no_motion.checked then begin end
    else if mouse_motion.checked then begin
      mot_damping   := (1-((mot_b_damping.position) / (mot_b_damping.max+1)));
    end else if ran_fluid_motion.checked then begin
      mot_damping   := (((mot_b_damping.position) / (mot_b_damping.max+1)));
      movex_timeout := 0;
      movey_timeout := 0;
    end else if ran_jump_motion.checked then begin
      movex_timeout := mot_ran_freq.max-mot_ran_freq.position
    end else if par_motion.checked then with central_poly do begin
      x_count  := 0;
      x_inc    := 2*mot_par_a.position;
      x_count2 := 0;
      x_inc2   := 2*mot_par_b.position;
      x_rad    := ((mot_par_c.position) / mot_par_c.max);
      y_count  := 0;
      y_inc    := 2*mot_par_d.position;
      y_count2 := 0;
      y_inc2   := 2*mot_par_e.position;
      y_rad    := ((mot_par_f.position) / mot_par_f.max);
      xc := xmid + round((xmid*x_rad)*cosine[x_count]*cosine[x_count2]);
      yc := ymid + round((ymid*y_rad)*sine  [y_count]*cosine[y_count2]);
    end;

    if hide_cursor.checked then screen.cursor := 2
    else screen.cursor := 1;

  end;
end;

destructor zonerings.destroy;
var lp1 : word;
begin
  dispose_256_pens(App_pens);
  for lp1:=0 to num_polys do listpoly[lp1].Free;
  inherited destroy;
end;

procedure zonerings.draw_all_rings(adc:hDC);
var lp1    : word;
    tempDC : HDC;
begin
  tempDC  := zone_DC;
  zone_DC := adc;
  if not Settings_Form.draw_vertices.checked then
    for lp1:=num_polys downto 1 do listpoly[lp1].drawpoly
  else
    for lp1:=num_polys downto 1 do listpoly[lp1].drawvert;
  zone_DC := tempDC;
end;

procedure zonerings.cyclerings;
var lp1  : integer;
    rubbingout,aspectsync : boolean;
begin
  with Settings_Form do with central_poly do begin
    rubbingout := rub_out.checked;
    aspectsync := aspect_sync.checked;
    if not draw_vertices.checked then begin
      for lp1:=num_polys downto 1 do begin
        if (rubbingout and listpoly[lp1-1].changed) then
          listpoly[lp1].erasepoly;
        listpoly[lp1].copy_from_below(listpoly[lp1-1]);
        if aspectsync then begin
          listpoly[lp1].xasp  := central_poly.xasp;
          listpoly[lp1].yasp  := central_poly.yasp;
        end;
        listpoly[lp1].drawpoly;
      end;
    end else begin
      for lp1:=num_polys downto 1 do begin
        if (rubbingout and listpoly[lp1-1].changed) then
          listpoly[lp1].erasevert;
        listpoly[lp1].copy_from_below(listpoly[lp1-1]);
        if aspectsync then begin
          listpoly[lp1].xasp  := central_poly.xasp;
          listpoly[lp1].yasp  := central_poly.yasp;
        end;
        listpoly[lp1].drawvert;
      end;
    end;

{ do we need to update the innermost/outermost zonering }

    if aspect_flag then begin
      aspect_countx  := (aspect_countx+aspect_incx) and tablesize;
      aspect_county  := (aspect_county+aspect_incy) and tablesize;
      xasp  := 1-(x_asp*(0.5+(sine[aspect_countx] / 2)));
      yasp  := 1-(y_asp*(0.5+(sine[aspect_county] / 2)));
      changed := true;
    end;

    if shape_per.checked then begin
      inc(shape_counter);
      if shape_counter>shape_timeout then begin
        shape_counter := 0;
        sid           := sides;
        changed       := true;
      end;
    end else if random(100)<shape_timeout then begin
      sid     := sides;
      changed := true;
    end;

    if no_rotation.checked then begin end
    else if sin_rotation.checked then begin
      rot      := rot + rot_inc;
      rot_loop :=(rot_loop + rot_inc2) and tablesize;
      vel      := round(sine[rot_loop]*rot_ampl) shl 16;
      changed  := true;
    end
    else begin
      if rot_ran_per.checked then begin
        inc(rotat_counter);
        if rotat_counter>rotat_timeout then begin
          rotat_counter := 0;
          rot := random(tablesizep1) shl 16;
          vel := (random(tot_rot)+min_rot_speed.position)*(8+random(8)) shl 16;
          changed := true;
        end;
      end else if rot_ran_ran.checked then begin
        if random(100)<rotat_timeout then begin
          rot := random(tablesizep1) shl 16;
          vel := (random(tot_rot)+min_rot_speed.position)*(8+random(8)) shl 16;
          changed := true;
        end;
      end;
    end;

    col := col + col_inc;
    if      col>=col_max then col:=col-col_max
    else if col<0        then col:=col+col_max;

    if no_motion.checked then begin end
    else if mouse_motion.checked then begin
      xc      := round(xc+((mouseX-xc)*mot_damping));
      yc      := round(yc+((mouseY-yc)*mot_damping));
      changed := true;
    end else if ran_fluid_motion.checked then begin
      inc(movex_counter);
      if movex_counter>movex_timeout then begin
        lastx         := xc;
        if ran_uniform.checked then diffx :=(random(2*xmid))-xc
        else                        diffx :=xmid-xc + (round(gauss_mean_sigma(0,sigmax))*ranbit);
        movex_timeout := system.Abs(round(diffx*mot_damping));
        if movex_timeout>0 then begin
          movex_counter := 0;
          move_tempx    := tablesize div movex_timeout;
        end else begin
          movex_counter := 1;
          move_tempx    := tablesize;
        end;
      end;
      inc(movey_counter);
      if movey_counter>movey_timeout then begin
        lasty         := yc;
        if ran_uniform.checked then diffy :=(random(2*ymid))-yc
        else                        diffy :=ymid-yc + (round(gauss_mean_sigma(0,sigmay))*ranbit);
        movey_timeout := system.Abs(round(diffy*mot_damping));
        if movey_timeout>0 then begin
          movey_counter := 0;
          move_tempy    := tablesize div movey_timeout;
        end else begin
          movey_counter := 1;
          move_tempy    := tablesize;
        end;
      end;
      changed := true;
      xc := lastx + round(diffx*sigmoids[movex_counter*move_tempx and tablesize]);
      yc := lasty + round(diffy*sigmoids[movey_counter*move_tempy and tablesize]);
    end
    else if ran_jump_motion.checked then begin
      inc(movex_counter);
      if movex_counter>movex_timeout then begin
        movex_counter := 0;
        if ran_uniform.checked then begin
          xc := (random(2*xmid));
          yc := (random(2*ymid));
        end else begin
          xc := xmid-(round(gauss_mean_sigma(0,sigmax))*ranbit);
          yc := ymid-(round(gauss_mean_sigma(0,sigmay))*ranbit);
        end;
        changed       := true;
      end;
    end else if par_motion.checked then begin
      x_count  := (x_count+x_inc) and tablesize;
      y_count  := (y_count+y_inc) and tablesize;
      x_count2 := (x_count2+x_inc2) and tablesize;
      y_count2 := (y_count2+y_inc2) and tablesize;
      xc := xmid + round((xmid*x_rad)*cosine[x_count]*cosine[x_count2]);
      yc := ymid + round((ymid*y_rad)*sine  [y_count]*cosine[y_count2]);
      changed  := true;
    end;

  end
end;

function zonerings.sides : word;
var lp1,prob : word;
begin
  prob:=random(total_prob)+1;
  for lp1:=0 to 9 do begin
    if prob<=densities[lp1] then begin sides:=lp1+1; exit; end
  end;
  sides:=0;
end;

procedure zonerings.setup_densities;
var lp1 : Integer;
begin
  total_prob:=0;
  for lp1:=0 to 9 do begin
    total_prob    := total_prob -
      TScrollbar(Settings_Form.FindComponent('density'+IntToStr(lp1+1))).position;
    densities[lp1]:= total_prob;
  end;
end;
{ ----------------------------------------------------------------------- }
{                       Form routines                                     }
{ ----------------------------------------------------------------------- }
procedure TZonewindow.WMSysCommand(var message:TWMSysCommand);
begin
  if message.CmdType=SC_Screensave then message.result:=0;
end;

procedure TZonewindow.FormResize(Sender: TObject);
begin
  SetBounds(0,0,screen.width,screen.height);
end;

procedure TZonewindow.FormCreate(Sender: TObject);
var lp1 : integer;
begin
  SetBounds(0,0,screen.width,screen.height);
  paused        := false;
  dialog_active := false;

  Xmid := ((GetSystemMetrics(SM_CXSCREEN)-1) div 2);
  ymid := ((GetSystemMetrics(SM_CYSCREEN)-1) div 2);
  sigmax := (xmid) div 4;
  sigmay := (ymid) div 4;
  if xmid>ymid then screen_:=ymid
  else              screen_:=xmid;

  frame_speed   := frame_speed_obj.Create;
  frame_speed.reset(100);

  pal_pos      := 0;
  pal_inc      := 5000*longint(Settings_Form.palette_inc.position);

  zone_instance := zonerings.Create;

  SelectPalette(canvas.handle,hpalApp,false);
  RealizePalette(canvas.handle);
  for lp1:=0 to 255 do cols[lp1] := PaletteIndex(lp1);

  want_speed := 1/StrToInt(Settings_Form.designed_speed.Caption);
  frame_speed.average_frame_speed := 1/want_speed;
  one_second := EncodeTime(0,0,1,0);
  last_time := Time;
end;

procedure TZonewindow.FormDestroy(Sender: TObject);
begin
  frame_speed.Free;
end;

procedure TZonewindow.NewSetting;
var lp1 : integer;
begin
  PatBlt(canvas.handle,0,0,width,height,BLACKNESS);
  zone_instance.Free;
  zone_instance := zonerings.Create;

  SelectPalette(canvas.handle,hpalApp,false);
  RealizePalette(canvas.handle);
  for lp1:=0 to 255 do cols[lp1] := PaletteIndex(lp1);
  pal_pos      := 0;
  pal_inc      := 5000*longint(Settings_Form.palette_inc.position);
  want_speed := 1/StrToInt(Settings_Form.designed_speed.Caption);
  frame_speed.average_frame_speed := 1/want_speed;
  speed_ok := false;
end;

function TZonewindow.are_settings_different(CheckStrings:TStringList) : boolean;
var someStrings : TStringList;
    lp1         : integer;
begin
  result := false;
  somestrings := nil;
  Settings_Form.WriteText(somestrings,'Temp setting');
  if somestrings.count=CheckStrings.count then begin
    for lp1:=0 to somestrings.count-1 do begin
      if somestrings[lp1]<>CheckStrings[lp1] then begin
        result := true;
        break;
      end;
    end;
  end else result := true;
  somestrings.Free;
end;

procedure TZonewindow.FormClick(Sender: TObject);
var TempStrings : TStringList;
    tempbool   : boolean;
    ddd : double;
    lp1 : integer;
begin
  dialog_active := true;
  Settings_Form.f_speed.caption := IntToStr(round(frame_speed.average_frame_speed));

  TempStrings := nil;
  Settings_Form.WriteText(TempStrings,'Temp setting');
  if Settings_Form.ShowModal=mrCancel then Settings_Form.ReadText(TempStrings)
  else begin
    if Options_Form.Run_mode.Checked and (not Settings_Form.just_loaded) and are_settings_different(tempStrings) then begin
      case MessageDlg('You have changed some settings, but are running in Auto-cycling mode'#13#10+
                      '              Changes will be lost when mode cycles.'#13#10#13#10+
        'Do you want to disable cycling ?', mtConfirmation, [mbYes,mbNo], 0) of
        mrNo     : ;
        mrYes    : Options_Form.Run_mode.Checked := false;
      end
    end;
    TempStrings.Free;
    NewSetting;
    paused    := false;
    last_time := Time;
  end;

  dialog_active := false;
end;

procedure TZonewindow.IdleAction(Sender:TObject; var Done:Boolean);
var adc        : HDC;
    temp,delay : integer;
    anim_pal   : palentries;
    animate    : boolean;
    sp_temp    : double;
begin
  zone_DC := Canvas.handle;

  pal_pos := pal_pos + pal_inc;
  temp    := smallint(HiWord(pal_pos));
  pal_pos := pal_pos - 65536*temp;

  if dialog_active then animate := false
  else if paused and Settings_Form.pause_freeze.checked then animate:=false
  else animate:=true;

  if (not animate) or (temp=0) then begin end
  else begin
    GetPaletteEntries(hpalApp,10,pal_colours,anim_pal[10]);
    if temp>0 then begin
      animatepalette(hpalApp,10,pal_colours-temp,@anim_pal[10+temp]);
      animatepalette(hpalApp,10+pal_colours-temp,temp,@anim_pal[10]);
    end
    else begin temp:=-temp;
      animatepalette(hpalApp,10+temp,pal_colours-temp,@anim_pal[10]);
      animatepalette(hpalApp,10,temp,@anim_pal[10+pal_colours-temp]);
    end;
  end;

  SelectPalette(zone_DC,hpalApp,false);
  RealizePalette(zone_DC);

  if (not dialog_active) then begin
    if Settings_Form.fix_speed.checked then begin
      speed_ok := false;
      while (tick_seconds2<want_speed) do begin
        if (want_speed-tick_seconds2)>0.02 then sleep(10);
        speed_ok := true;
      end;
      sp_temp := tick_seconds2;
    end;
    start_ticker;
    if (not paused) then begin
      zone_instance.CycleRings;
    end;
    stop_ticker;
    if Settings_Form.fix_speed.checked then frame_speed.Update(1/sp_temp)
    else frame_speed.Update(1/tick_seconds);
  end;

  if paused then done := Settings_Form.pause_mouse.checked
  else done := false;

  if (not paused) and (not dialog_active) and Options_Form.Run_mode.Checked then with Options_Form do begin
    time_now := Time;
    if time_now-last_time>(one_second*mintime) then begin
      if time_now-last_time>(one_second*maxtime) then begin
        LoadSave_Form.select_random_setting(cycle_tagged.Checked);
        NewSetting;
        last_time := time_now;
      end else begin
        if random(maxtime*64)=1 then begin
          LoadSave_Form.select_random_setting(cycle_tagged.Checked);
          NewSetting;
          last_time := time_now;
        end;
      end;
    end;
  end;

end;

procedure TZonewindow.FormMouseDown(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
begin
  case button of
    mbRight : paused := not paused;
    mbLeft  : ;
  end;
end;

procedure TZonewindow.FormPaint(Sender: TObject);
begin
  if About_Form.Visible then exit;
  if zone_instance<>nil then zone_instance.draw_all_rings(canvas.handle);
end;

procedure TZonewindow.FormMouseMove(Sender: TObject; Shift: TShiftState; X,
  Y: Integer);
begin
  MouseX := X;
  MouseY := Y;
end;

function TZonewindow.GetPalette : hPalette;
begin
  if hPalApp<>0 then result := hPalApp
  else result := inherited GetPalette;
end;

end.
