///////////////////////////////////////////////////////////////////////////////
//                                                                           //
//                           Zonerings Options form                          //
//                             John Biddiscombe                             //
//                           J.Biddiscombe@rl.ac.uk                          //
//                                  Freeware                                 //
//                                                                           //
///////////////////////////////////////////////////////////////////////////////
unit OptionsForm;

interface

uses
  { Borland }
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  Thinkbox, Buttons, BorBtns, ExtCtrls, ComCtrls, StdCtrls,
  { Downloaded }
  Ubag;

type
  TOptions_Form = class(TForm)
    OK_btn: TSpeedButton;
    Panel2: TPanel;
    Start_mode: TBorCheck;
    Run_mode: TBorCheck;
    Panel1: TPanel;
    Label1: TLabel;
    Label2: TLabel;
    cycle_tagged: TBorRadio;
    cycle_all: TBorRadio;
    min_time: TEdit;
    max_time: TEdit;
    UpDown1: TUpDown;
    UpDown2: TUpDown;
    Label3: TLabel;
    UseCompression: TBorRadio;
    SaveText: TBorRadio;
    Settings_file_name: TEdit;
    Label4: TLabel;
    Zone_ini: TBag;
    Label5: TLabel;
    Label6: TLabel;
    procedure FormShow(Sender: TObject);
    procedure OK_btnClick(Sender: TObject);
    procedure min_timeExit(Sender: TObject);
    procedure max_timeExit(Sender: TObject);
    procedure UpDown2Changing(Sender: TObject; var AllowChange: Boolean);
    procedure FormDestroy(Sender: TObject);
    procedure Settings_file_nameChange(Sender: TObject);
    procedure UseCompressionClick(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
    mintime : integer;
    maxtime : integer;
    procedure ReadIni;
  end;

var
  Options_Form: TOptions_Form;

implementation

uses LoadSaveForm;

{$R *.DFM}

procedure TOptions_Form.FormShow(Sender: TObject);
var err : integer;
begin
  CenterWindow(Handle);
  Val(min_time.Text,mintime,err);
  Val(max_time.Text,maxtime,err);
end;

procedure TOptions_Form.OK_btnClick(Sender: TObject);
begin
  SendMessage(min_time.handle,CM_EXIT,0,0);
  SendMessage(max_time.handle,CM_EXIT,0,0);
  if (mintime>0) and (maxtime>0) and (mintime<=maxtime) then ModalResult := mrOK
  else showmessage('Min and Max time settings are invalid'#13#10+
                   '    (1->3600 allowed) and Min<Max');
end;

procedure TOptions_Form.min_timeExit(Sender: TObject);
var err : integer;
begin
  Val(min_time.Text,mintime,err);
  if mintime<1 then ActiveControl := min_time;
end;

procedure TOptions_Form.max_timeExit(Sender: TObject);
var err : integer;
begin
  Val(max_time.Text,maxtime,err);
  if maxtime<1 then ActiveControl := max_time;
end;

procedure TOptions_Form.UpDown2Changing(Sender: TObject;
  var AllowChange: Boolean);
begin
  if UpDown2.Position<=UpDown1.position then AllowChange:=false;
end;

procedure TOptions_Form.ReadIni;
var err : integer;
begin
  with Zone_ini do begin
    IniFile := ExtractFilePath(Application.Exename) + 'Zone_Rings_32_Config.ini';
    ReadIni;
    Start_mode.Checked      := GetBoolean('Start_auto_cycling',True);
    Run_mode.Checked        := Start_mode.Checked;
    Cycle_tagged.Checked    := GetBoolean('Tagged_settings_only',True);
    cycle_all.Checked       := not Cycle_tagged.Checked;
    min_time.text           := IntToStr(GetInteger('Min_Time',30));
    max_time.text           := IntToStr(GetInteger('Max_Time',60));
    UseCompression.Checked  := GetBoolean('Use_Compression',True);
    SaveText.Checked        := not UseCompression.Checked;
    Settings_file_name.Text := GetString ('Zone_settings_file','Zone_Rings_32_Settings');
  end;
  Val(min_time.Text,mintime,err);
  Val(max_time.Text,maxtime,err);
end;

procedure TOptions_Form.FormDestroy(Sender: TObject);
begin
  with Zone_ini do begin
    IniFile := ExtractFilePath(Application.Exename) + 'Zone_Rings_32_Config.ini';
    SetBoolean('Start_auto_cycling',Start_mode.Checked);
    SetBoolean('Tagged_settings_only',Cycle_tagged.Checked);
    SetInteger('Min_Time',StrToInt(min_time.text));
    SetInteger('Max_Time',StrToInt(max_time.text));
    SetBoolean('Use_Compression',UseCompression.Checked);
    SetString ('Zone_settings_file',Settings_file_name.Text);
    WriteIni;
  end;
end;

procedure TOptions_Form.Settings_file_nameChange(Sender: TObject);
begin
  Settings_file_name.Text := Trim(Settings_file_name.Text);
  if Assigned(LoadSave_Form) then LoadSave_Form.modifications := true;
end;

procedure TOptions_Form.UseCompressionClick(Sender: TObject);
begin
  // ini file sets this before it is created.
  if Assigned(LoadSave_Form) then LoadSave_Form.modifications := true;
end;

end.
