///////////////////////////////////////////////////////////////////////////////
//                                                                           //
//              Zone Rings from Outer-Space 32 bit Delphi 2 version          //
//                         John Biddiscombe Jan 1997                        //
//                           J.Biddiscombe@rl.ac.uk                          //
//                                  Freeware                                 //
//                                                                           //
///////////////////////////////////////////////////////////////////////////////
{

Thanks to
 Enrico Lodolo CIS 100275,1255 - Internet ldlc18k1@bo.nettuno.it
 For the Borland BWCC style RadioButtons, and Radiocheckboxes

 Original TBag (INI File) code by Mike Orriss (3K Computer Consultancy):
 Compuserve: 100570,121

 LZH compression
 * LZSS coded by Haruhiko OKUMURA
 * Adaptive Huffman Coding coded by Haruyasu YOSHIZAKI
 * Edited and translated to English by Kenji RIKITAKE
 * Translated from C to Turbo Pascal by Douglas Webb   2/18/91
 *    Update and bug correction of TP version 4/29/91 (Sorry!!)
 *    Added Delphi exception handling may 09 1996 Danny Heijl
 *                                                Danny.Heijl@cevi.be

}

program Zone_Rings_32;

uses
  { Borland }
  Windows,
  Controls, 
  StdCtrls,
  SysUtils,
  Forms,
  Math,
  Graphics,
  Dialogs,
  { Mine }
  ThinkBox,
  PalUnit,
  Math_utils,
  Main_window in 'Main_window.pas' {Zonewindow},
  SettingsForm in 'SettingsForm.pas' {Settings_Form},
  LoadSaveForm in 'LoadSaveForm.pas' {LoadSave_Form},
  OptionsForm in 'OptionsForm.pas' {Options_Form},
  AboutForm in 'AboutForm.pas' {About_Form};

{$R *.RES}

{$R Zone_Rings_32_Cursors.res }

var
  screenDC        : hDC;
  bits_pixel      : integer;
  xnow,ynow       : integer;
  current_display : TDeviceMode;
  display_changed : boolean;
  forced          : boolean;

function find_256_graphics_mode : boolean;
var lp1         : integer;
    dm          : TDeviceMode;
    found       : boolean;
    s           : string;
    new_display : TDeviceMode;
begin
  lp1  := 0; result := false;
  dm.dmSize := SizeOf(TDeviceMode);
  // test once to find current display settings
  while EnumDisplaySettings(nil,lp1,dm) do begin
    if (bits_pixel=dm.dmBitsPerPel) and (xnow=dm.dmPelsWidth) and (ynow=dm.dmPelsHeight) then current_display := dm;
    inc(lp1);
  end;
  // now look for a compatible 8 bit mode
  lp1 := 0; found := false;
  while EnumDisplaySettings(nil,lp1,dm) do begin
    if (dm.dmBitsPerPel=8) and (xnow=dm.dmPelsWidth) and (ynow=dm.dmPelsHeight) then begin
      found := true;
      new_display := dm;
    end;
    inc(lp1);
  end;
  s := '  Zonerings32 uses palette animation to achieve colour cycling effects'#13+
       '                     you need to use a 256 colour display mode.       '#13+
       '     Do you wish to temporarily switch to a 256 colour display mode ? ';
  case MessageDlg(s, mtConfirmation, [mbYes,mbNo], 0) of
    mrYes : begin
      if not found then begin
        showmessage('Couldn''t find a compatible display mode');
        exit;
      end;
      new_display.dmFields := DM_BITSPERPEL + DM_PELSWIDTH + DM_PELSHEIGHT;
      case ChangeDisplaySettings(new_display,0) of
        DISP_CHANGE_SUCCESSFUL	: display_changed := true;
        DISP_CHANGE_RESTART : begin
          s := '  Windows requested a restart - do you wish to try and force a colour change ? '#13+
               'This may cause your windows colours to be badly affected until your next reboot';
          case MessageDlg(s, mtConfirmation, [mbYes,mbNo], 0) of
            mrYes : case ChangeDisplaySettings(new_display,$80000000 { undocumented CDS_FORCE }) of
              DISP_CHANGE_SUCCESSFUL : begin
                display_changed := true;
                forced          := true;
                result          := true;
              end;
              else exit;
            end;
            else exit;
          end;
        end;
        else begin
          ChangeDisplaySettings(current_display,0); // put it back to normal
          showmessage('Couldn''t set display mode - or machine would need restarting');
          exit;
        end
      end;
    end;
    mrNo  : exit;
  end;
end;

function CheckProcessor : boolean;
var si : TSystemInfo;
begin
  GetSystemInfo(si);
  if si.dwProcessorType<586 then begin
    showmessage( '           Zonerings32 uses floating point arithmetic, and is intended    '#13+
                 '             to be run on a Pentium processor based machine or better.    '#13#13+
                 ' please see the accompanying text file to get the 486 version, which uses '#13+
                 '       integer fixed point arithmetic and is faster on 386/486 machines.  ');
    result := false;
  end
  else result := true;
end;

begin
  randomize;
  display_changed := false;
  forced          := false;
  CheckProcessor;
  screenDC      := GetDC(GetDesktopWindow);
  bits_pixel    := GetDeviceCaps(screenDC,BITSPIXEL);
  xnow          := GetDeviceCaps(screenDC,HORZRES);
  ynow          := GetDeviceCaps(screenDC,VERTRES);
  ReleaseDC(GetDesktopWindow,screenDC);
  if bits_pixel<>8 then begin
    if not find_256_graphics_mode then exit;
  end;
  if GetSystemMetrics(SM_CXVSCROLL)>=20 then begin
    showmessage( '  Zonerings32 makes heavy use of scrollbars   '#13+
                 '    this machine has the scrollbars set to an   '#13+
                 ' unusually large value. This may impair control ');
  end;
  Thinking(nil,'Initializing');
    screen.cursors[1] := LoadCursor(hInstance, pChar('MY_CURS_1'));
    screen.cursors[2] := LoadCursor(hInstance, pChar('MY_CURS_2'));
    Application.Initialize;
    Application.Title := 'Zonerings from Outer-Space';
  // Create these two first, but make zonewindow the mainform.
  Application.HelpFile := 'Zone_Rings_32_Help.hlp';
  setup_tables;
  Settings_Form  := TSettings_Form.Create(Application);
  Options_Form   := TOptions_Form.Create(Application);
  Options_Form.ReadIni;
  stopped_thinking;
  LoadSave_Form := TLoadSave_Form.Create(Application);
  Thinking(nil,'Initializing second phase');
  Application.CreateForm(TZonewindow, Zonewindow);
  Application.CreateForm(TAbout_Form, About_Form);
  stopped_thinking;
  Application.OnIdle := Zonewindow.IdleAction;
  Application.Run;
  if display_changed then begin
    if forced then ChangeDisplaySettings(current_display,$80000000)
    else ChangeDisplaySettings(current_display,0);
  end;
end.
